<?php

namespace Spec\Infomaker\Everyware\Support;

use Infomaker\Everyware\Support\Str;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class StrSpec extends ObjectBehavior
{
    public function it_is_initializable() {
        $this->shouldHaveType( Str::class );
    }

    public function it_can_append_to_string_with_a_given_glue() {
        $string = 'original string';

        $this->append( 'this will be appended', $string, ' ' )->shouldReturn('original string this will be appended');
        $this->append( 'this will be appended', $string, ' - ' )->shouldReturn('original string - this will be appended');
    }

    public function it_can_append_int_to_string_with_a_given_glue() {
        $string = "10";

        $this->append( ' - 10', $string, '' )->shouldReturn('10 - 10');
        $this->append( '10', $string, ' - ' )->shouldReturn('10 - 10');
    }

    public function it_can_convert_utf_8_string_to_ascii() {
        $this->ascii( 'Här förändras bokstäverna på ett sätt som passar. Vilken bra idé!' )->shouldReturn( 'Har forandras bokstaverna pa ett satt som passar. Vilken bra ide!' );
    }

    public function it_can_convert_string_to_camel_case() {
        $camel_case         = 'stringToConvert';
        $strings_to_convert = [
            'string to convert',
            'string_to_convert',
            'string-to-convert',
            'StringToConvert',
            'String To Convert',
            'String_To_Convert',
            'String-To-Convert'
        ];

        foreach ( $strings_to_convert as $string ) {
            $this->camel( $string )->shouldReturn( $camel_case );
        }
    }

    public function it_can_convert_string_to_slug_case() {

        $slug_case          = 'string-to-convert';
        $strings_to_convert = [
            'string to convert',
            'string_to_convert',
            'string-to-convert',
            'String To Convert',
            'String_To_Convert',
            'String-To-Convert'
        ];

        foreach ( $strings_to_convert as $string ) {
            $this->slug( $string )->shouldReturn( $slug_case );
        }

        $this->slug( 'StringToConvert' )->shouldReturn( 'stringtoconvert' );
    }

    public function it_can_convert_string_to_slug_with_optional_seperator() {

        $slug_case          = 'string_to_convert';
        $strings_to_convert = [
            'string to convert',
            'string_to_convert',
            'string-to-convert',
            'String To Convert',
            'String_To_Convert',
            'String-To-Convert'
        ];

        foreach ( $strings_to_convert as $string ) {
            $this->slug( $string, '_' )->shouldReturn( $slug_case );
        }
        $this->slug( 'StringToConvert' )->shouldReturn( 'stringtoconvert' );
    }

    public function it_can_convert_string_to_snake_case() {
        $snake_case         = 'string_to_convert';
        $strings_to_convert = [
            'string to convert',
            'StringToConvert',
            'String To Convert',
        ];

        foreach ( $strings_to_convert as $string ) {
            $this->snake( $string )->shouldReturn( $snake_case );
        }
    }

    public function it_can_convert_string_to_snake_case_with_optional_seperator() {
        $snake_case         = 'string-to-convert';
        $strings_to_convert = [
            'string to convert',
            'StringToConvert',
            'String To Convert'
        ];

        foreach ( $strings_to_convert as $string ) {
            $this->snake( $string, '-' )->shouldReturn( $snake_case );
        }
    }

    public function it_can_convert_string_to_studly_case() {
        $studly_case        = 'StringToConvert';
        $strings_to_convert = [
            'string to convert',
            'string_to_convert',
            'string-to-convert',
            'String To Convert',
            'String_To_Convert',
            'String-To-Convert'
        ];

        foreach ( $strings_to_convert as $string ) {
            $this->studly( $string )->shouldReturn( $studly_case );
        }
    }

    public function it_can_convert_softhyphens_into_hex() {
        $str = 'this text has a softhyphen'; // Includes the char x0085 between 'soft' and 'hyphen'
        $this->convertSoftHyphen( $str )->shouldReturn('this text has a soft&shy;hyphen');
    }

    public function it_can_convert_string_to_upper_case() {
        $this->upper( 'lowercase' )->shouldReturn( 'LOWERCASE' );
        $this->upper( 'LowerCase' )->shouldReturn( 'LOWERCASE' );
        $this->upper( 'lower_case' )->shouldReturn( 'LOWER_CASE' );
    }

    public function it_can_convert_string_to_lower_case() {
        $this->lower( 'UPPERCASE' )->shouldReturn( 'uppercase' );
        $this->lower( 'UpperCase' )->shouldReturn( 'uppercase' );
        $this->lower( 'upper_case' )->shouldReturn( 'upper_case' );
    }

    public function it_can_convert_string_to_title_case() {
        $this->title( 'title text' )->shouldReturn( 'Title Text' );
        $this->title( 'TITLE TEXT' )->shouldReturn( 'Title Text' );
        $this->title( 'titleText' )->shouldReturn( 'Titletext' );
    }

    public function it_can_make_first_character_uppercase() {
        $this->ucfirst( 'title text' )->shouldReturn( 'Title text' );
        $this->ucfirst( 'title Text' )->shouldReturn( 'Title Text' );
    }

    public function it_can_match_if_string_contains_substring() {
        $this->contains( 'String to be searched through', 'searched' )->shouldBe( true );
        $this->contains( 'String to be [searched] through', '[searched]' )->shouldBe( true );
        $this->contains( 'String to be searched through', 'missing' )->shouldBe( false );
    }

    public function it_can_match_if_string_contains_sentence() {
        $this->contains( 'String to be searched through', 'to be searched' )->shouldBe( true );
    }

    public function it_can_match_if_string_contains_multiple_substrings() {
        $this->contains( 'String to be searched through', [ 'searched', 'be' ] )->shouldBe( true );
        $this->contains( 'String to be [searched] through', [ '[searched]', 'be' ] )->shouldBe( true );
        $this->contains( 'String to be searched through', [ 'missing', 'word' ] )->shouldBe( false );
    }

    public function it_can_match_if_string_starts_with_substring() {
        $this->startsWith( 'String to be searched through', 'String' )->shouldBe( true );
        $this->startsWith( 'String to be searched through', 'searched' )->shouldBe( false );
    }

    public function it_can_match_if_string_starts_with_sentence() {
        $this->startsWith( 'String to be searched through', 'String to be searched' )->shouldBe( true );
    }

    public function it_can_match_if_string_starts_with_one_of_given_substrings() {
        $this->startsWith( 'String to be searched through', [
            'through',
            'searched',
            'String',
            'be'
        ] )->shouldBe( true );
        $this->startsWith( 'String to be searched through', [ 'searched', 'be', 'to' ] )->shouldBe( false );
    }

    public function it_can_match_if_string_ends_with_substring() {
        $this->endsWith( 'String to be searched through', 'through' )->shouldBe( true );
        $this->endsWith( 'String to be searched through', 'searched' )->shouldBe( false );
    }

    public function it_can_match_if_string_ends_with_sentence() {
        $this->endsWith( 'String to be searched through', 'to be searched through' )->shouldBe( true );
    }

    public function it_can_match_if_string_ends_with_one_of_given_substrings() {
        $this->endsWith( 'String to be searched through', [ 'through', 'searched', 'to', 'be' ] )->shouldBe( true );
        $this->endsWith( 'String to be searched through', [ 'searched', 'be', 'to' ] )->shouldBe( false );
    }

    public function it_can_match_if_string_is_empty() {
        // Considered empty
        $this->isEmpty( '' )->shouldBe( true );
        $this->isEmpty( '   ' )->shouldBe( true );
        $this->isEmpty( ' ' )->shouldBe( true );

        // Considered Not empty
        $this->isEmpty( 'String' )->shouldBe( false );
        $this->isEmpty( '0' )->shouldBe( false );
        $this->isEmpty( '1' )->shouldBe( false );
        $this->isEmpty( 0 )->shouldBe( false );
        $this->isEmpty( 1 )->shouldBe( false );
    }

    public function it_can_match_if_string_is_not_empty() {
        // Considered empty
        $this->notEmpty( '' )->shouldBe( false );
        $this->notEmpty( '   ' )->shouldBe( false );
        $this->notEmpty( ' ' )->shouldBe( false );

        // Considered Not empty
        $this->notEmpty( 'String' )->shouldBe( true );
        $this->notEmpty( '0' )->shouldBe( true );
        $this->notEmpty( '1' )->shouldBe( true );
        $this->notEmpty( 0 )->shouldBe( true );
        $this->notEmpty( 1 )->shouldBe( true );
    }

    public function it_can_match_a_given_string_to_a_given_pattern() {
        $this->is( 'a match', 'a match' )->shouldBe( true );
        $this->is( 'a/matched/url', 'a/matched/url' )->shouldBe( true );
    }

    public function it_can_match_a_given_string_with_a_wildcard() {
        $this->is( 'path/to/*', 'path/to/something' )->shouldBe( true );
        $this->is( 'namespace\*', 'namespace\class' )->shouldBe( true );
        $this->is( 'file.*', 'file.php' )->shouldBe( true );
        $this->is( '*.php', 'file.php' )->shouldBe( true );
    }

    public function it_can_make_sure_a_given_string_dose_not_match_a_given_pattern() {
        $this->not( 'not a match', 'a match' )->shouldBe( true );
        $this->not( 'a/matched/url', 'a/matched/url' )->shouldBe( false );
    }

    public function it_can_make_sure_a_given_string_dose_not_match_a_given_pattern_using_widcard() {
        $this->not( 'path/to/*', 'path/to/something' )->shouldBe( false );
        $this->not( 'path/to/some*', 'path/to/something' )->shouldBe( false );
        $this->not( 'path/to/some/*', 'path/to/something' )->shouldBe( true );
        $this->not( 'namespace\*', 'namespace\class' )->shouldBe( false );
        $this->not( 'files.*', 'file.php' )->shouldBe( true );
        $this->not( '*.php', 'file.css' )->shouldBe( true );
    }

    public function it_can_calculate_the_length_of_a_string() {
        $this->length( 'this is a string with 35 characters' )->shouldBe( 35 );
        $this->length( ' this is a string with 35 characters ' )->shouldBe( 37 );
        $this->length( '    this is a string with 35 characters ' )->shouldBe( 40 );
    }

    public function it_can_count_the_number_of_words_in_a_given_string() {
        $this->wordCount( 'this is a string has seven words' )->shouldBe( 7 );
        $this->wordCount( 'this is a string has 7 words' )->shouldBe( 7 );
    }

    // Manipulations:
    // ======================================================

    public function it_can_cap_a_string_with_a_single_instance_of_a_given_value() {
        $this->finish( 'value/cap/of/single/instance', '/' )->shouldReturn( 'value/cap/of/single/instance/' );
        $this->finish( 'value/cap/of/single/instance', '/cap' )->shouldReturn( 'value/cap/of/single/instance/cap' );
        $this->finish( 'value cap of single instance', '_' )->shouldReturn( 'value cap of single instance_' );

        // Should Not add extra if cap already exists
        $this->finish( 'value/cap/of/single/instance/', '/' )->shouldReturn( 'value/cap/of/single/instance/' );
        $this->finish( 'value cap of single instance_', '_' )->shouldReturn( 'value cap of single instance_' );
        $this->finish( 'value/cap/of/single/instance/cap', '/cap' )->shouldReturn( 'value/cap/of/single/instance/cap' );

    }

    public function it_can_limit_the_number_of_characters_in_a_string() {
        $this->limit( 'this is a string with 35 characters', 30 )->shouldReturn( 'this is a string with 35 chara...' );
    }

    public function it_can_limit_the_number_of_characters_in_a_string_with_optional_end() {
        $this->limit( 'this is a string with 35 characters', 30, '' )->shouldReturn( 'this is a string with 35 chara' );
        $this->limit( 'this is a string with 35 characters', 30, '*' )->shouldReturn( 'this is a string with 35 chara*' );
    }

    public function it_can_limit_the_number_of_characters_in_a_string_without_end_if_not_effected() {
        $this->limit( 'this is a string with 35 characters', 40 )->shouldReturn( 'this is a string with 35 characters' );
        $this->limit( 'this is a string with 35 characters', 35 )->shouldReturn( 'this is a string with 35 characters' );
    }

    public function it_can_limit_the_number_of_words_in_a_string() {
        $this->words( 'this is a string with 7 words', 5 )->shouldReturn( 'this is a string with...' );
        $this->words( 'this is a [string-block] with 7 words', 5 )->shouldReturn( 'this is a [string-block] with...' );
        $this->words( 'this is a [string-block] with 7 words', 4 )->shouldReturn( 'this is a [string-block]...' );
    }

    public function it_can_limit_the_number_of_words_in_a_string_with_optional_end() {
        $this->words( 'this is a string with 7 words', 5, '' )->shouldReturn( 'this is a string with' );
        $this->words( 'this is a string with 7 words', 5, '/' )->shouldReturn( 'this is a string with/' );
    }

    public function it_can_limit_the_number_of_words_in_a_string_without_end_if_not_effected() {
        $this->words( 'this is a string with 7 words', 8 )->shouldReturn( 'this is a string with 7 words' );
        $this->words( 'this is a string with 7 words', 7 )->shouldReturn( 'this is a string with 7 words' );
    }

    public function it_can_parse_string_into_method_callback() {
        $this->parseCallback( 'Class@Method', '' )->shouldReturn( [ 'Class', 'Method' ] );
        $this->parseCallback( 'Namespace\Class@Method', '' )->shouldReturn( [ 'Namespace\Class', 'Method' ] );
        $this->parseCallback( 'Class', 'Method' )->shouldReturn( [ 'Class', 'Method' ] );
        $this->parseCallback( 'Namespace\Class', 'Method' )->shouldReturn( [ 'Namespace\Class', 'Method' ] );
    }

    public function it_can_prepend_to_a_string_with_a_given_glue() {
        $string = 'original string';

        $this->prepend( 'this will be prepended to', $string, ' ' )->shouldReturn('this will be prepended to original string');
        $this->prepend( 'this will be prepended to', $string, ' - ' )->shouldReturn('this will be prepended to - original string');
    }

    public function it_can_prepend_int_to_string_with_a_given_glue() {
        $string = '10';

        $this->prepend( '10 - ', $string, '' )->shouldReturn('10 - 10');
        $this->prepend( '10', $string, ' - ' )->shouldReturn('10 - 10');
    }

    public function it_can_generate_a_random_string_of_given_length() {
        $random_string = $this->quickRandom( 25 );
        $this->length( $random_string )->shouldBe( 25 );
    }

    public function it_can_generate_a_random_string_of_words_equal_to_given_int() {
        $random_string = $this->quickLorem( 25 );
        $this->wordCount( $random_string )->shouldBe( 25 );
    }

    public function it_can_grab_a_postion_of_the_string() {
        $this->substr( 'This is a string with 35 characters', 5 )->shouldReturn( 'is a string with 35 characters' );
        $this->substr( 'This is a string with 35 characters', 5, 10 )->shouldReturn( 'is a strin' );
    }

    public function it_can_strip_tags_with_allowence_of_given_list() {
        $string = "<p>This is a string with a <b>bold word</b> and a <a href='#'>link</a> and an <i class='icon-class'>icon</i></p>";
        $this->stripTags( $string )->shouldReturn('This is a string with a bold word and a link and an icon');
        $this->stripTags( $string, '<p><b>' )->shouldReturn('<p>This is a string with a <b>bold word</b> and a link and an icon</p>');
    }

    public function it_can_truncate_a_string_into_a_given_length_with_cap() {
        $string = 'Lorem ipsum dolor sit amet, adipiscing elit consectetur. Curabitur venenatis massa non ante condimentum, quis eleifend mauris tempus.';

//        $this->truncate($string, 50)->shouldReturn("Lorem ipsum dolor sit amet, adipiscing elit consectetur...");
//        $this->truncate($string, 50, ['break_word' => true])->shouldReturn("Lorem ipsum dolor sit amet, adipiscing elit consec...");
//        $this->truncate($string, 45)->shouldReturn("Lorem ipsum dolor sit amet, adipiscing elit...");
    }

    public function it_can_truncate_a_string_into_a_given_length_without_cap() {
        $string = 'Lorem ipsum dolor sit amet, adipiscing elit consectetur.';

//        $this->truncate($string, 60)->shouldReturn("Lorem ipsum dolor sit amet, adipiscing elit consectetur.");
//        $this->truncate($string, 40, ['cap' => ''])->shouldReturn("Lorem ipsum dolor sit amet, adipiscing elit");
    }
}
